{**
 DelphiPI (Delphi Package Installer)
 Author  : ibrahim dursun (t-hex) thex [at] thexpot ((dot)) net
 License : GNU General Public License 2.0
**}
unit PackageCompiler;

interface
uses JclBorlandTools, PackageInfo, PackageList, SysUtils, Classes, CompilationData, ProgressMonitor;

type

   TPackageCompiler = class
   private
     fCompilationData: TCompilationData;
     fCancel: boolean;
     fSourceFilePaths: TStringList;
     fExtraOptions: String;
     fAllPaths: TStringList;
     FEnvPaths: TStringList;

     function ConvertToShortPaths(const paths : TStringList): string;
     function GetInstallation: TJclBorRADToolInstallation;
     function GetPackageList: TPackageList;
     procedure AddFileToDCUPath;
     function SetEnvPath: boolean;
     function GetEnvPath: boolean;
    procedure Compiling;
    procedure Installing;
   protected
     procedure PrepareExtraOptions; virtual;
     procedure ResolveHelpFiles;
     procedure ResolveSourcePaths; virtual;
     function CompilePackage(const Info : TPackageInfo): Boolean; virtual;
     function ReCompilePackage(const Info: TPackageInfo):boolean; virtual;
     function InstallPackage(const Info : TPackageInfo;
                addFlag:boolean): Boolean; virtual;

     function AddEnvPath(const path:string):boolean;

     property Installation: TJclBorRADToolInstallation read GetInstallation;
     property PackageList: TPackageList read GetPackageList;
     property Data: TCompilationData read fCompilationData;
   public
     constructor Create(const compilationData: TCompilationData); virtual;
     destructor Destroy; override;
     procedure Compile; virtual;
     //Properties
     property Cancel: boolean read fCancel write fCancel;
     property SourceFilePaths: TStringList read fSourceFilePaths write fSourceFilePaths;
     property AllPaths: TStringList read fAllPaths;
     property ExtraOptions: string read fExtraOptions;
     property EnvPaths:TStringList read FEnvPaths;
   end;

implementation

uses JclFileUtils, JclStrings,jclShell,JclSysInfo,Windows,Utils;
{ TPackageCompiler }

function TPackageCompiler.GetEnvPath:boolean;
var
 Path:string;
begin
  result:=GetGlobalEnvironmentVariable('Path',Path);
  if result
   then FEnvPaths.Text:=StringReplace(Path,';',#13#10,[rfReplaceAll])
   else FEnvPaths.Text:='';
end;

function  TPackageCompiler.SetEnvPath:boolean;
var
 Path:string;
begin
  Path:=StringReplace(FEnvPaths.Text,#13#10,';',[rfReplaceAll]);
  SetGlobalEnvironmentVariable('Path',Path);
  result:=GetEnvPath;
end;

constructor TPackageCompiler.Create(const compilationData: TCompilationData);
var
 Path:string;
 reWriteEnv:boolean;
 i : Integer;
begin
  fCompilationData := compilationData;
  fSourceFilePaths := TStringList.Create;
  fAllPaths := TStringList.Create;
  fAllPaths.Delimiter := ';';

  FEnvPaths := TStringList.Create;
  FEnvPaths.Delimiter := ';';
  FEnvPaths.Duplicates:= dupIgnore;
  FEnvPaths.Text:='';
  if not GetEnvPath then exit;
  reWriteEnv:=false;
  i:=0;
  while i < EnvPaths.Count do begin
   Path:=EnvPaths[i];
   if DirectoryExists(Path)
    then inc(i)
    else begin
     FEnvPaths.Delete(i);
     reWriteEnv:=true;
    end;
  end;
  if not reWriteEnv then exit;
  SetEnvPath;
end;

destructor TPackageCompiler.Destroy;
begin
  fSourceFilePaths.Free;
  fAllPaths.Free;
  FEnvPaths.Free;
  inherited;
end;

function TPackageCompiler.GetInstallation: TJclBorRADToolInstallation;
begin
  Result := Data.Installation;
end;

function TPackageCompiler.GetPackageList: TPackageList;
begin
  Result := Data.PackageList;
end;

function TPackageCompiler.AddEnvPath(const path: string): boolean;
var
 it:string;
begin
 for it in EnvPaths do begin
  result:=LowerCase(it)=LowerCase(path);
  if result then exit;
 end;
 EnvPaths.Add(path);
 result:=SetEnvPath;
end;

procedure TPackageCompiler.AddFileToDCUPath;
  function ConvertToCurVerName(OldVerName:string):string;
  var
   suffix:string;
   VerOrd:integer;
   changed:boolean;
  begin
   verOrd:= Ord(oldvername[Length(oldvername)]);
   changed:=false;
   while (verOrd>=$30)and(verOrd<=$39) do begin
     oldvername:=StrLeft(oldVerName,Length(oldvername)-1);
     verOrd:= Ord(oldvername[Length(oldvername)]);
     changed:=true;
   end;
   suffix:=self.Installation.VersionNumberStr;
   if changed
    then result:=StrLeft(oldVerName,Length(oldvername)-1)+suffix
    else result:=oldvername;
  end;
var
  files,Exts: TStringList;
  Path,DCUPath: string;
  TagFileName,Ext,SRCFileName:string;
  PackageName:string;
  i: Integer;
  dontCopy:boolean;
begin
  assert(assigned(Data));
  files := TStringList.Create;
  files.Sorted := true;
  files.Duplicates := dupIgnore;

  exts:=TStringList.Create;
  exts.Sorted := true;
  exts.Duplicates := dupIgnore;
  exts.Add('.dpk');
  exts.Add('.bpk');
  exts.Add('.bpl');
  exts.Add('.pas');
  exts.Add('.hpp');
  exts.Add('.h');
  exts.Add('.c');
  exts.Add('.cpp');
  exts.Add('.bat');
  exts.Add('.bak');
  exts.Add('.tvsconfig');
  exts.Add('.todo');
  exts.Add('.exe');
  exts.Add('.com');
  exts.Add('.cfg');
  exts.Add('.mak');
  exts.Add('.rc');
  exts.Add('.manifest');
  exts.Add('.dproj');
  exts.Add('.identcache');
  exts.Add('.txt');
  exts.Add('.cmd');
  exts.Add('.bpg');
  exts.Add('.bdsgroup');
  exts.Add('.groupproj');
  exts.Add('.local');
  exts.Add('.rsp');
  exts.Add('.db');
  exts.Add('.map');
  exts.Add('.');
  exts.Add('..');
  try
   for i:=0 to PackageList.Count-1
    do if PackageList[i].Status=psSuccess then
    for Path in PackageList[i].SourceFilePaths
     do AdvBuildFileList(PathAppend(path,'*.*'),faAnyFile,files,amAny,
           [flFullnames, flRecursive],'', nil);

   DCUPath:=PathAddSeparator(Data.DCUOutputFolder);

   if files.Count =0 then exit;

   for SRCFileName in files do begin
     ext:=LowerCase(ExtractFileExt(SRCFileName));
     dontCopy:=(exts.IndexOf(ext)>=0)or(Pos('~',ext)>0);
     if dontCopy then continue;
     if ext='.res' then begin
       PackageName:=ConvertToCurVerName(PathExtractFileNameNoExt(SRCFileName));
       dontCopy:=PackageList.IndexOf(PackageName)>=0;
       if dontCopy then continue;
     end;
     TagFileName:=DCUPath+ExtractFileName(SRCFileName);
     FileCopy(SRCFileName,TagFileName,false);
   end;
  finally
    files.Free;
    exts.Free;
  end;
end;

procedure TPackageCompiler.Compile;
begin
 if data.Mode=coInstall
  then Installing
  else if (data.Mode=coFull)or(data.Mode=coCompile)
   then Compiling;
end;

procedure TPackageCompiler.Installing;
var
  addDCPPath,addDCUPath:boolean;
  info: TPackageInfo;
  hasInstalled:boolean;
  RTInstalled:boolean;
begin
  addDCPPath:=DirectoryExists(Data.DCPOutputFolder);
  addDCUPath:=DirectoryExists(Data.DCUOutputFolder);

  PackageList.SortList;
  hasInstalled:=false;

  for info in PackageList do
  begin
    if info.RunOnly then RTInstalled:=true
     else if InstallPackage(info,false)then hasInstalled:=true;
    if fCancel then break;
  end;

  installation.AddToLibrarySearchPath(Data.BPLOutputFolder);

  if RTInstalled then AddEnvPath(Data.BPLOutputFolder);
  if addDCUPath then installation.AddToLibrarySearchPath(Data.DCUOutputFolder);
  if addDCPPath then installation.AddToLibrarySearchPath(Data.DCPOutputFolder);
end;

procedure TPackageCompiler.Compiling;
 function addBPLToBin(BPLPath:string):boolean;
 var
   BINPath,BDSBPLPath:string;
 begin
   BINPath:=LowerCase(PathAddSeparator(installation.BinFolderName));
   BDSBPLPath:=LowerCase(PathAddSeparator(Installation.BPLOutputPath));
   result:=(BPLPath<>BDSBPLPath)and(BINPath<>BPLPath);
 end;
var
  i: integer;
  info: TPackageInfo;
  compilationSuccessful     : boolean;
  CanInstall                : boolean;
  addSRCPath,addDCUPath     : boolean;
  addFileToBin : boolean;
  hasDoinstall,Compiled:boolean;
  BPLPath,DCUPath:string;
  RTInstalled:boolean;
begin
  if not DirectoryExists(Data.BPLOutputFolder)
   then CreateDirectory(PChar(Data.BPLOutputFolder),nil);
  if not DirectoryExists(Data.DCPOutputFolder)
   then CreateDirectory(PChar(Data.DCPOutputFolder),nil);
  if not DirectoryExists(Data.DCUOutputFolder)
   then CreateDirectory(PChar(Data.DCUOutputFolder),nil);

  if SourceFilePaths.Count = 0 then
    ResolveSourcePaths;

  PrepareExtraOptions;

  DCUPath:=LowerCase(PathAddSeparator(Data.DCUOutputFolder));
  BPLPath:=LowerCase(PathAddSeparator(Data.BPLOutputFolder));

  addSRCPath :=Length(DCUPath)=0;
  addFileToBin:=addBPLToBin(BPLPath);

  if Data.HelpFiles.Count = 0
   then ResolveHelpFiles;

  PackageList.SortList;
  hasDoinstall:=false;
  RTInstalled:=false;
  Compiled:=false;
  for i := 0 to PackageList.Count - 1 do
  begin
    info := PackageList[i];
    compilationSuccessful := CompilePackage(info);
    if compilationSuccessful then compiled:=true;
    CanInstall:=(Data.Mode=coFull) and compilationSuccessful;
    if CanInstall
     then begin
      if info.RunOnly then  RTInstalled:=true
       else if InstallPackage(info,addSRCPath and not info.RunOnly)
          then hasDoinstall:=true;
     end;
    if fCancel then break;
  end;

  if hasDoinstall and addFileToBin
   then installation.AddToLibrarySearchPath(Data.BPLOutputFolder);
  if RTInstalled then AddEnvPath(Data.BPLOutputFolder);
  addDCUPath:= (DCUPath<>BPLPath)and not addSRCPath
         and (hasDoinstall or RTInstalled);
  if addDCUPath
   then installation.AddToLibrarySearchPath(Data.DCUOutputFolder);
  if not addSRCPath and Compiled then AddFileToDCUPath;
end;

function TPackageCompiler.ReCompilePackage(const Info: TPackageInfo):boolean;
var
   packDir:string;
   Found:boolean;
   files: TStringList;
   Path,filename: string;
begin
  files := TStringList.Create;
  files.Sorted := true;
  files.Duplicates := dupIgnore;
  result:=false;
  try
   packDir:=ExtractFileDir(info.filename);
   found:=info.SourceFilePaths.IndexOf(packDir)>0;
   if found then exit;
   files.Clear;
   for path in info.SourceFilePaths do
     AdvBuildFileList(PathAppend(path,'*.*'),faAnyFile,files
            ,amAny,[flFullnames, flRecursive],'', nil);

   for path in files do fileCopy(path,packdir,false);

   try
    Result := Installation.DCC32.MakePackage(
                Info.filename,
                Data.BPLOutputFolder,
                Data.DCPOutputFolder,
                fExtraOptions);
   finally

    for path in files
      do begin
       filename:=PathAppend(packDir,ExtractFileName(path));
       if fileExists(filename)
        then if not SHDeleteFiles(0, FileName, [doSilent, doFilesOnly])
         then FileDelete(filename,true);
      end;
   end;
  finally
   files.Free;
  end;
end;

function TPackageCompiler.CompilePackage(
  const Info: TPackageInfo): Boolean;
begin
  Result := Installation.DCC32.MakePackage(
                Info.filename,
                Data.BPLOutputFolder,
                Data.DCPOutputFolder,
                fExtraOptions);
  if result then exit;
  result:=ReCompilePackage(info);
end;

procedure TPackageCompiler.PrepareExtraOptions;
var
  shortPaths: string;
  I: Integer;
begin
  fAllPaths.Clear;
  ExtractStrings([';'],[' '],PWideChar(Installation.LibrarySearchPath),fAllPaths);
  fAllPaths.AddStrings(SourceFilePaths);
  fAllPaths.Add(Installation.BPLOutputPath);

  for I := 0 to fAllPaths.Count - 1 do
    fAllPaths[i] := Installation.SubstitutePath(StrTrimQuotes(fAllPaths[i]));

  shortPaths := ConvertToShortPaths(fAllPaths);
  fExtraOptions := '-B -Q -CC';
  fExtraOptions := fExtraOptions + ' -I'+shortPaths;
  fExtraOptions := fExtraOptions + ' -U'+shortPaths;
  fExtraOptions := fExtraOptions + ' -O'+shortPaths;
  fExtraOptions := fExtraOptions + ' -R'+shortPaths;
  if Data.DCUOutputFolder<>''
   then fExtraOptions := fExtraOptions + ' -N0'+PathGetShortName(Data.DCUOutputFolder);
  if Length(Data.Conditionals) > 0 then
    fExtraOptions := fExtraOptions + ' -D'+Data.Conditionals;
end;

function TPackageCompiler.ConvertToShortPaths(const paths : TStringList):string;
var
  path : string;
begin
  Result := '';
  for path in paths do
    Result := Result + StrDoubleQuote(PathGetShortName(path)) + ';';
end;

function TPackageCompiler.InstallPackage(
  const Info: TPackageInfo;addFlag:boolean): Boolean;
  function InstallDesign(BPLFileName:string):boolean;
  var
   Path:string;
  begin
   Result := Installation.RegisterPackage(BPLFileName, info.Description);
   if not result then exit;
   for path in info.SourceFilePaths
    do  installation.AddToLibraryBrowsingPath(path);
   if addFlag
    then for path in info.SourceFilePaths
     do installation.AddToLibrarySearchPath(path);
  end;
var
  BPLFileName,BPLName : string;
begin
  result:=info.RunOnly;
  if result then exit;
  BPLName:=PathExtractFileNameNoExt(Info.FileName)
         + Info.Suffix + '.bpl';
  BPLFileName :=PathAddSeparator(Data.BPLOutputFolder)+BPLName;
  result:=InstallDesign(BPLFileName);
end;

procedure TPackageCompiler.ResolveSourcePaths;
var
  i: integer;
  path: string;
begin
  Assert(assigned(SourceFilePaths));
  SourceFilePaths.Clear;
  SourceFilePaths.Sorted := true;
  SourceFilePaths.Duplicates := dupIgnore;
  for i := 0 to PackageList.Count - 1 do
  begin
    SourceFilePaths.Add(ExtractFileDir(PackageList[i].FileName));
    for path in PackageList[i].SourceFilePaths
     do SourceFilePaths.Add(path);
  end;
end;

procedure TPackageCompiler.ResolveHelpFiles;
var
  files: TStringList;
  filename: string;
begin
  assert(assigned(Data));

  files := TStringList.Create;
  try
    AdvBuildFileList(Data.BaseFolder +'\*.hlp',
           faAnyFile,
           files,
           amAny,
           [flFullnames, flRecursive],
           '', nil);
     for filename in files do
       Data.HelpFiles.Add(filename);
  finally
    files.Free;
  end;
end;

end.
