{**
 DelphiPI (Delphi Package Installer)
 Author  : ibrahim dursun (t-hex) thex [at] thexpot ((dot)) net
 License : GNU General Public License 2.0
**}
unit PageConfig;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, PageBase, StdCtrls, ExtCtrls, WizardIntfs, JclBorlandTools, ActnList,
  DB, ADODB;

type
  TFrmConfig = class(TWizardPage)
    grpBaseFolder: TGroupBox;
    btnRefresh: TButton;
    lError: TLabel;
    edtUserName: TLabeledEdit;
    edtPWD: TLabeledEdit;
    edtValid: TLabeledEdit;
    chkEnODBC: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure btnRefreshClick(Sender: TObject);
    procedure chkEnODBCClick(Sender: TObject);
  private
    fInstalled:boolean;
    function Installed: boolean;
    function CreateAccessDSN(const FullDbFileName, UserName,
      Password: string): boolean;
    function ExistsDSN(FullDbFileName: string): boolean;
    function EnableAlarmODBC(const value:boolean=true): boolean;
    function getAlarmODBCAddr: Int64;
    function OpenSCUFile: TFileStream;
    function getAlarmODBC:boolean;
    function LocalPath: string;
    function IFIXInstalled: boolean;
    function PDBPath: string;
    function AppPath: string;
  protected
    function GetCanShowPage: boolean;override;
  public
    procedure UpdateWizardState; override;
  end;

implementation

uses gnugettext, FormWizard,Utils,JCLFileUtils,JclRegistry,JclShell;
{$R *.dfm}

const
 dbFileName='dbHistory.mdb';
 ODBCCfgFileName='AlmODBC.Cfg';
 DSNName='IFIXReport';
 DriverName='Microsoft Access Driver (*.mdb)';

 ODBCKey='Software\ODBC\ODBC.INI';
 ODBCDsKey='Software\ODBC\ODBC.INI\ODBC Data Sources';
 RegDrvName='software\ODBC\ODBCINST.INI\'+DriverName;
 DSNKey='Software\ODBC\ODBC.INI\'+DSNName;
 DBQ='DBQ';
 Driver='Driver';
 DriverId='DriverId';
 FIL='FIL';
 PWD='PWD';
 SafeTransactions='SafeTransactions';
 UID='UID';
 JetKey='Engines\Jet';

 IFIXPDBKEY='Software\FixDMACS\PDBPATH';
 IFIXAPPKEY='Software\FixDMACS\APPPATH';
 IFIXLOCKEY='Software\FixDMACS\LOCPATH';
 IFIXSCUKEY='Software\FixDMACS\SCUFILE';
 IFIXKEY='Software\FixDMACS';

function TFrmConfig.ExistsDSN(FullDbFileName:string): boolean;
var
  dbqValue: String;
begin
  result:=RegValueExists(HKEY_CURRENT_USER,ODBCDsKey,DsnName);
  if not result then begin
    ShowInfo(format(_('%s not found in %s'),[DsnName,ODBCDsKey]));
    exit;
  end;

  result:=RegKeyExists(HKEY_CURRENT_USER,DSNKey);
  if not result then begin
    ShowInfo(format(_('%s can not open'),[DSNKey]));
    exit;
  end;

  dbqValue:=RegReadString(HKEY_CURRENT_USER,DSNKey,DBQ);
  dbqValue:=LowerCase(PathGetLongName(dbqValue));
  FullDbFileName:=LowerCase(Trim(FullDbFileName));
  result:=dbqValue=FullDbFileName;
  if not result then begin
    ShowInfo(format(_('Current DbFile(%s) not int Registry(%s)'),
            [FullDbFileName,DSNKey+'\'+DBQ]));
    exit;
  end;

  edtUserName.Text:=RegReadString(HKEY_CURRENT_USER,DSNKey,UID);
  edtPWD.Text:=RegReadString(HKEY_CURRENT_USER,DSNKey,PWD);
  edtValid.Text:=edtPWD.Text;
end;

procedure TFrmConfig.chkEnODBCClick(Sender: TObject);
begin
 EnableAlarmODBC(chkEnODBC.Checked);
end;

function TFrmConfig.CreateAccessDSN(const FullDbFileName,UserName,Password:string): boolean;
var
  sTmp: String;
begin
  sTmp:=RegReadString(HKEY_LOCAL_MACHINE,RegDrvName,'driver');
  result:=sTmp<>'';
  if not result then exit;

  result:=RegValueExists(HKEY_CURRENT_USER,ODBCDsKey,DsnName);
  if not result
   then RegWriteString(HKEY_CURRENT_USER,ODBCDsKey,DsnName,DriverName);

  result:=RegCreateKey(HKEY_CURRENT_USER,DSNKey)=ERROR_SUCCESS;
  if not result then exit;

  RegWriteString(HKEY_CURRENT_USER,DSNKey,DBQ,FullDbFileName);
  RegWriteString(HKEY_CURRENT_USER,DSNKey,Driver,sTmp);
  RegWriteInteger(HKEY_CURRENT_USER,DSNKey,DriverId,25);
  RegWriteString(HKEY_CURRENT_USER,DSNKey,FIL,'MS Access;');
  RegWriteString(HKEY_CURRENT_USER,DSNKey,PWD,Password);
  RegWriteInteger(HKEY_CURRENT_USER,DSNKey,SafeTransactions,0);
  RegWriteString(HKEY_CURRENT_USER,DSNKey,UID,UserName);

  result:=RegCreateKey(HKEY_CURRENT_USER,JetKey)=ERROR_SUCCESS;
  if not result then exit;

  RegWriteString(HKEY_CURRENT_USER,JetKey,'ImplicitCommitSync','');
  RegWriteInteger(HKEY_CURRENT_USER,JetKey,'MaxBufferSize',800);
  RegWriteInteger(HKEY_CURRENT_USER,JetKey,'PageTimeout',5);
  RegWriteInteger(HKEY_CURRENT_USER,JetKey,'Threads',3);
  RegWriteString(HKEY_CURRENT_USER,JetKey,'UserCommitSync','Yes');
end;

function TFrmConfig.getAlarmODBCAddr:Int64;
//Enabled: 2283 -04 23ef-(80h or 23ef),highest bit=1
//Disable: 2283 -00 23ef-(7Fh and 23ef)=7fh
const
 BuffLen=4096;
 flagLen=8;
 AlarmODBCFLAG:array[0..flagLen-1] of Byte=($FF,$FF,$01,$00,$84,$00,$03,$03);
 function compare(value:array of byte):boolean;
 var
  i:Integer;
 begin
  for I := 0 to flagLen-1
   do begin
    result:=AlarmODBCFLAG[i]=value[i];
    if not result then break;
   end;
 end;
 function Search(var Buff:array of byte;highOffet:Int64):Int64;
 var
  value:array[0..flagLen-1] of Byte;
  Offset,j:Integer;
 begin
   result:=-1;
   for offset := 0 to highOffet
   do begin
    for j := 0 to flagLen - 1
     do value[j]:=Buff[Offset+j];
    if not compare(value) then continue;
    result:=Offset+flagLen;
    break;
   end;
 end; 
 function Found(fs:TFileStream;var Buff:array of Byte):Int64;
 var
  offset,inx,count:Int64;
 begin
  fs.Seek(0,soBeginning);
  count:=(fs.Size+flagLen) div bufflen;
  result:=-1;
  while count>0 do begin
   dec(Count);
   offset:=fs.Position;
   fs.ReadBuffer(Buff,Bufflen);
   inx:=Search(Buff,Bufflen-flagLen);
   if inx>=0 then begin
    result:=offset+inx;
    break;
   end;
   fs.Seek(fs.Position-flagLen+1,soBeginning);
  end;
  if result>=0 then exit;
  count:=fs.Size-fs.Position-1;
  if count>0 then begin
   offset:=fs.Position;
   fs.ReadBuffer(Buff,count);
   inx:=Search(Buff,Count-flagLen);
   if inx>=0 then result:=inx+offset;
  end;
 end;
var
 fs:TFileStream;
 Buff:array[0..BuffLen-1]of Byte;
begin
  result:=-1;
  fs:=OpenSCUFile;
  if not Assigned(fs) then exit;
  try
   result:=found(fs,Buff);
  finally
   fs.Free;
  end;
end;

function TFrmCOnfig.OpenSCUFile:TFileStream;
var
 SCUName:string;
begin
  SCUName :=RegReadStringDef(HKEY_CLASSES_ROOT,IFIXSCUKEY,'','');
  result:=nil;
  if not FileExists(SCUName) then begin
   ShowInfo(format(_('IFix SCU File(%s) Error'),[SCUName]));
   exit;
  end;
  result:=TFileStream.Create(SCUName,fmOpenReadWrite or fmShareDenyWrite);
  if not Assigned(result) then begin
   ShowInfo(format(_('Can''t open the SCU File(%s),close iFix to try again!'),[SCUName]));
   exit;
  end;
end;

function TFrmConfig.EnableAlarmODBC(const value:boolean):boolean;
const
 FLAG21=$80;
 FLAG20=$7F;
var
 fs:TFileStream;
 Buff:array[0..3] of Byte;
 flagAddr:Int64;
 AlarmCfgFile,LocalPath:string;
begin
  result:=false;
  flagAddr:=self.getAlarmODBCAddr;
  if flagAddr<0 then exit;

  fs:=OpenSCUFile;
  if not Assigned(fs) then exit;
  try
   fs.Seek(flagAddr,soBeginning);
   fs.ReadBuffer(Buff,1);
   if value
    then Buff[0]:=FLAG21 or Buff[0]
    else Buff[0]:=FLAG20 and Buff[0];
   fs.Seek(flagAddr,soBeginning);
   fs.WriteBuffer(Buff,1);
  AlarmCfgFile:=PathAppend(LocalPath,ODBCCfgFileName);
  result:=FileExists(AlarmCfgFile);
  if not result
   then begin
    result:=LoadResourceToFile('CFG',AlarmCfgFile);
    if not result then begin
     ShowInfo(format(_('Error,Alarm ODBC Config File %s not found in LOCAL folder(%s)!')
      ,[ODBCCfgFileName,LocalPath]));
     exit;
    end;
   end;
   result:=true;
  finally
   fs.Free;
  end;
end;

function TFrmConfig.getAlarmODBC:boolean;
const
 FLAG21=$80;
 FLAG20=$7F;
var
 fs:TFileStream;
 Buff:array[0..3] of Byte;
 flagAddr:Int64;
begin
  result:=false;
  flagAddr:=self.getAlarmODBCAddr;
  if flagAddr<0 then exit;
  fs:=OpenSCUFile;
  if not Assigned(fs) then exit;
  try
   fs.Seek(flagAddr,soBeginning);
   fs.ReadBuffer(Buff,1);
   result:=(FLAG21 and Buff[0])=FLAG21;
  finally
   fs.Free;
  end;
end;

function TFrmConfig.LocalPath:string;
begin
 result:=LowerCase(PathGetLongName(RegReadStringDef(HKEY_CLASSES_ROOT,IFIXLOCKEY,'','')));
end;

function TFrmConfig.AppPath:string;
begin
 result:=LowerCase(PathGetLongName(RegReadStringDef(HKEY_CLASSES_ROOT,IFIXAPPKEY,'','')));
end;

function TFrmConfig.PDBPath:string;
begin
 result:=LowerCase(PathGetLongName(RegReadStringDef(HKEY_CLASSES_ROOT,IFIXPDBKEY,'','')));
end;

function TFrmConfig.IFIXInstalled:boolean;
begin
  result:=RegKeyExists(HKEY_CLASSES_ROOT,IFIXKEY);
  lError.Visible:=not result;
  if not result then ShowInfo(_('IFix not found'));
end;

function TFrmConfig.Installed:boolean;
var
 curDbFile,
 PDBExeName,
 ExeName,langPath: string;
 adoConn:TADOConnection;
begin
  fInstalled:=false;
  Result:=IFIXInstalled;
  if not result then exit;

  result:=DirectoryExists(LocalPath)
      and DirectoryExists(PDBPath);

  if not result then begin
   ShowInfo(_('IFix current Path of PDB and Local Error'));
   Exit;
  end;

  ExeName:=ExtractFileName(Application.ExeName);
  PDBExeName:=LowerCase(PathAppend(PDBPath,ExeName));

  result:=PDBExeName=LowerCase(Application.ExeName);
  if not result
   then begin
    if not FileCopy(application.exename,PDBExeName,true)
     then ShowInfo(
       format(_('Please place %s in project''s PDB folder(%s)!'),
       [ExeName,PDBPath]))
     else begin
       langPath:=PathAppend(ExtractFileDir(Application.ExeName),'locale');
       CopyDirectory(langPath,PDBPath);
       ShellExecEx(PDBExeName);
       application.Terminate;
     end;
     exit;
   end;

  curDbFile:=PathAppend(PDBPath,dbFileName);
  result:=FileExists(curDbFile);
  if not result
   then begin
    result:=LoadResourceToFile('DB',curDbFile);
    if not result then begin
     ShowInfo(format(_('Error,%s not found in PDB folder(%s)!'),
      [dbFileName,PDBPath]));
     exit;
    end;
   end;

//  result:=EnableAlarmODBC;
//  if not result then exit;
  chkEnODBC.Checked:=getAlarmODBC;

  result:=ExistsDSN(curDBFile);
  if not result then  exit;
  adoConn:=TADOConnection.Create(nil);
 try
  adoConn.LoginPrompt:=false;
  adoConn.KeepConnection:=false;
  if adoConn.Connected then adoConn.Connected:=false;
  adoConn.ConnectionString:=CreateConnStr(curDbFile,edtUserName.Text,edtPwd.Text);
  try
   adoConn.Connected:=true;
   result:=adoConn.Connected;
   if not result then ShowInfo(format(_('Can''t connect to %s'),[curDbFile]));
  except
   on E:exception do begin
    ShowInfo(e.Message);
    result:=false;
   end;
  end;
  if not result then exit;
  adoConn.Connected:=false;
  Wizard.SetStateValue('ConnStr',adoConn.ConnectionString);
  ShowInfo(_('Config successful'));
 finally
  FreeAndNil(adoConn);
 end;
 fInstalled:=result;
end;

procedure TFrmConfig.FormCreate(Sender: TObject);
const
 langPath='locale\zh_CN\LC_MESSAGES';
 langName='default.mo';
 procedure LoadDefaultLang(Path:string);
 begin
  LoadResourceToFile('CN',Path);
  if assigned(DefaultInstance)
   then FreeAndNil(DefaultInstance);
  DefaultInstance:=TGnuGettextInstance.Create;
 end;
var
 Path:string;
begin
 Path:=PathAppend(ExtractFileDir(application.ExeName),langPath);
 if DirectoryExists(Path) then begin
   Path:=PathAppend(Path,langName);
   if not FileExists(Path)
    then LoadDefaultLang(Path);
 end else
  if ForceDirectories(Path)
    then LoadDefaultLang(PathAppend(Path,langName));
 wizard.SetHeader(_('Config Database'));
 TranslateComponent(self);
end;

function TFrmConfig.GetCanShowPage: boolean;
begin
 result:=not Installed;
end;

procedure TFrmConfig.btnRefreshClick(Sender: TObject);
var
 curDbFile:string;
 CanClose:boolean;
begin
 curDbFile:=PathAppend(ExtractFileDir(Application.ExeName),dbFileName);
 CanClose:=self.edtPWD.Text=self.edtValid.Text;
 if not CanClose
  then begin
    ShowInfo(_('Valid is different password!'));
    exit;
  end;
 CanClose:=CreateAccessDSN(curDBFile,edtUserName.Text,edtPWD.Text);
 if not CanClose
  then begin
   ShowInfo( _('Error,Create ODBC DSN Error!'));
   exit;
  end;
 Installed;
 UpdateWizardState;
end;

procedure TFrmConfig.UpdateWizardState;
var
  action: TAction;
begin
  inherited;
  wizard.GetAction(wbtback).Enabled:=false;
  wizard.GetAction(wbtNext);
  action := wizard.GetAction(wbtNext);
  action.Enabled := fInstalled;
  grpBaseFolder.Enabled:=not (lError.Visible or action.Enabled);
  edtUserName.Enabled:=grpBaseFolder.Enabled;
  edtValid.Enabled:=grpBaseFolder.Enabled;
  edtPwd.Enabled:=grpBaseFolder.Enabled;
  btnRefresh.Enabled:=grpBaseFolder.Enabled;
end;

end.
